# -*- coding: utf-8 -*-
from ckeditor_link.templatetags import ckeditor_link_tags

try:
    reload
except NameError:
    from importlib import reload

from ckeditor_link.tests.utils.selenium_utils import SeleniumTestCase
from cms import api
from cms.constants import TEMPLATE_INHERITANCE_MAGIC
from django.test import Client, override_settings

from ckeditor_link.tests.test_app.models import TestModel
from ckeditor_link import conf

# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse


class ContribLinkModelTests(SeleniumTestCase):
    fixtures = ['test_app_link_model.json', ]

    def setUp(self):
        self.test_object = TestModel.objects.get(pk=101)
        self.test_object_cms_page = TestModel.objects.get(pk=102)
        self.page1 = api.create_page(
            slug='page1',
            title="Page1",
            language='en',
            template=TEMPLATE_INHERITANCE_MAGIC,
            parent=None
        )
        self.page1.publish('en')
        self.page2 = api.create_page(
            title="Page2",
            language='en',
            template=TEMPLATE_INHERITANCE_MAGIC,
            parent=None,
            position='first-child',
        )
        self.page2.publish('en')
        # page2 has id=3!
        super(ContribLinkModelTests, self).setUp()

    def tearDown(self):
        self.webdriver.quit()

    def test_admin_loads(self):
        """
        loading link forms
        """
        self.login()
        self.open(reverse('admin:test_app_contriblinkmodel_add'))
        self.webdriver.wait_for_css("#id_external", )
        self.open(reverse('admin:test_app_cmsfilerlinkmodel_add'))
        self.webdriver.wait_for_css("#id_cms_page_0", )

    def test_admin_page_fallback(self):
        """
        older versions have "page" instead of cms_page as data- attributes
        """
        self.login()
        self.open('{}?page={}'.format(
            reverse('admin:test_app_cmsfilerlinkmodel_add'),
            self.page2.id
        ))
        self.webdriver.wait_for_css("#id_cms_page_1 option", )
        option = self.webdriver.wait_for_css("#id_cms_page_1 option[selected]", )
        self.assertEqual(option.get_attribute('value'), str(self.page2.id))

    @override_settings(
        CKEDITOR_LINK_MODEL='ckeditor_link.tests.test_app.models.CMSFilerLinkModel'
    )
    def test_cms_page_link_fallback(self):
        reload(conf)
        reload(ckeditor_link_tags)
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object_cms_page.id])
        self.test_object_cms_page.richtext = self.test_object_cms_page.richtext.replace(
            '<cms_page_id_page2>', str(self.page2.id)
        )
        self.test_object_cms_page.save()
        response = client.get(url)
        self.assertContains(response, 'href="/page2/"')

    @override_settings(
        CKEDITOR_LINK_MODEL='ckeditor_link.tests.test_app.models.CMSFilerLinkModel',
        SITE_ID=2,
    )
    def test_cms_page_link_to_other_site(self):
        reload(conf)
        reload(ckeditor_link_tags)
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object_cms_page.id])
        self.test_object_cms_page.richtext = self.test_object_cms_page.richtext.replace(
            '<cms_page_id_page2>', str(self.page2.id)
        )
        self.test_object_cms_page.save()
        response = client.get(url)
        self.assertContains(response, 'href="//example.com/page2/"')



from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.wait import WebDriverWait
from selenium import webdriver
from selenium.webdriver.firefox.options import Options


# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse


class SeleniumTestCase(StaticLiveServerTestCase):
    """
    A base test case for Selenium, providing hepler methods for generating
    clients and logging in profiles.
    """

    username = 'admin'
    password = 'admin'

    def setUp(self):
        User.objects.create_superuser(self.username, 'admin@free.fr', self.password)
        # Instantiating the WebDriver will load your browser
        options = Options()
        if settings.HEADLESS_TESTING:
            options.add_argument("--headless")
        self.webdriver = CustomWebDriver(options=options, )

    def open(self, url):
        self.webdriver.get("%s%s" % (self.live_server_url, url))

    def login(self):
        self.open(reverse('admin:index'))
        # SeleniFIXTURESum knows it has to wait for page loads (except for AJAX requests)
        # so we don't need to do anything about that, and can just
        # call find_css. Since we can chain methods, we can
        # call the built-in send_keys method right away to change the
        # value of the field
        self.webdriver.find_css('#id_username').send_keys(self.username)
        # for the password, we can now just call find_css since we know the page
        # has been rendered
        self.webdriver.find_css("#id_password").send_keys(self.password)
        # You're not limited to CSS selectors only, check
        # http://seleniumhq.org/docs/03_webdriver.html for
        # a more compreehensive documentation.
        self.webdriver.find_element_by_xpath('//input[@type="submit"]').click()
        self.webdriver.wait_for_css("body.dashboard")


class CustomWebDriver(webdriver.Firefox):
    """Our own WebDriver with some helpers added"""

    def find_css(self, css_selector):
        """Shortcut to find elements by CSS. Returns either a list or singleton"""
        elems = self.find_elements_by_css_selector(css_selector)
        found = len(elems)
        if found == 1:
            return elems[0]
        elif not elems:
            raise NoSuchElementException(css_selector)
        return elems

    def wait_for_css(self, css_selector, timeout=4):
        """ Shortcut for WebDriverWait"""
        return WebDriverWait(self, timeout).until(lambda driver: driver.find_css(css_selector))

    def wait_for_iframe(self, iframe_selector, timeout=4):
        """ Shortcut for WebDriverWait"""
        return WebDriverWait(self, timeout).until(lambda driver: driver.frame_to_be_available_and_switch_to_it())



# coding: utf-8
from __future__ import unicode_literals
import re

from cms.models import CMSPlugin
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    dont use this yet, it assumes too many things of your existing installation.
    rather use it as starting point, and copy and paste it.
    """
    help = 'Check for djangocms-text-ckeditor subplugins, convert to django-ckeditor-link style'  # noqa

    def add_arguments(self, parser):
        parser.add_argument(
            'plugins',
            nargs='+',
            help='Plugins to check',
        )
        parser.add_argument(
            '--field',
            required=True,
            help='Field to check',
        )

    def handle(self, *args, **options):

        text_plugins = CMSPlugin.objects.filter(
            plugin_type__in=options['plugins']
        )
        for plugin in text_plugins:
            instance, plugin_class = plugin.get_plugin_instance()
            img_pattern = re.compile(r'<img.*?>')
            obj_pattern = re.compile(r'plugin_obj_[0-9]*')
            image_tags = re.findall(img_pattern, instance.body)
            if len(image_tags):
                for tag in image_tags:
                    obj_match = re.findall(obj_pattern, tag)
                    if len(obj_match):
                        plugin_id = obj_match[0].replace('plugin_obj_', '')
                        # print plugin_id
                        link_plugin = CMSPlugin.objects.get(pk=plugin_id)
                        # print link_plugin
                        link_instance, link_class = link_plugin.get_plugin_instance()
                        link_data = {
                            'link_text': link_instance.get_link_text(),
                            'link': link_instance.get_link(),
                            'link_type': link_instance.get_link_type(),
                            'page': link_instance.page_id,
                            'file': link_instance.file_id,
                            'external': link_instance.external_url,
                            'mailto': link_instance.mailto,
                            'phone': link_instance.phone,
                        }
                        for key, value in link_data.iteritems():
                            if not value:
                                link_data[key] = ''
                        new_link = '<a href="{link}"' \
                                   ' data-ckeditor-link="true"' \
                                   ' data-link_type="{link_type}"' \
                                   ' data-page="{page}"' \
                                   ' data-file="{file}"' \
                                   ' data-external="{external_url}"' \
                                   ' data-mailto="{mailto}"' \
                                   ' data-phone="{mailto}"' \
                                   '>' \
                                   '{link_text}</a>'.format(**link_data)
                        instance.body = instance.body.replace(tag, new_link)
                instance.save()



"""URLs to run the tests."""
from django.conf.urls import include, url
from django.contrib import admin

from ckeditor_link.tests.test_app.views import TestModelDetailView


urlpatterns = [
    url(r'^admin/', admin.site.urls),
    url(r'^testmodel/(?P<pk>\d+)/$', TestModelDetailView.as_view(), name='testmodel_detail'),
    url(r'^', include('cms.urls')),
]

# if settings.DEBUG:
#     urlpatterns += [
#         url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT}),
#     ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-07-17 03:17
from __future__ import unicode_literals

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('filer', '0010_auto_20180414_2058'),
        ('test_app', '0002_auto_20161128_1458'),
    ]

    operations = [
        migrations.CreateModel(
            name='CMSFilerLinkModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('link_type', models.CharField(blank=True, default='', max_length=20, verbose_name='Link type')),
                ('link_style', models.CharField(blank=True, default='', max_length=64, verbose_name='Link Style')),
                ('free', models.CharField(blank=True, default='', max_length=512, verbose_name='Internal / Other')),
                ('external', models.URLField(blank=True, default='', verbose_name='External Address')),
                ('mailto', models.EmailField(blank=True, default='', max_length=254, verbose_name='E-Mail')),
                ('phone', models.CharField(blank=True, default='', max_length=64, verbose_name='Phone')),
                ('cms_page', cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='cms.Page')),
                ('file', filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='filer.File')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContribLinkModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('link_type', models.CharField(blank=True, default='', max_length=20, verbose_name='Link type')),
                ('link_style', models.CharField(blank=True, default='', max_length=64, verbose_name='Link Style')),
                ('free', models.CharField(blank=True, default='', max_length=512, verbose_name='Internal / Other')),
                ('external', models.URLField(blank=True, default='', verbose_name='External Address')),
                ('mailto', models.EmailField(blank=True, default='', max_length=254, verbose_name='E-Mail')),
                ('phone', models.CharField(blank=True, default='', max_length=64, verbose_name='Phone')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.utils.translation import ugettext_lazy as _
from django.conf import settings


use = False
if 'cms' in settings.INSTALLED_APPS and 'filer' in settings.INSTALLED_APPS:
    use = True

CKEDITOR_LINK_USE_CMS_FILER = getattr(
    settings,
    'CKEDITOR_LINK_USE_CMS_FILER',
    use
)

CKEDITOR_LINK_STYLE_CHOICES = getattr(
    settings, 'CKEDITOR_LINK_STYLE_CHOICES', (
        ('', _("Default")),
        ('button', _("Button")),
    )
)

if CKEDITOR_LINK_USE_CMS_FILER:
    CKEDITOR_LINK_TYPE_CHOICES = getattr(
        settings, 'CKEDITOR_LINK_TYPE_CHOICES', (
            ('', _("None")),
            ('cms_page', _("Page")),
            ('file', _("File")),
            ('free', _("Other / Free")),
            ('external', _("External URL")),
            ('mailto', _("E-Mail")),
            ('phone', _("Phone")),
        )
    )
else:
    CKEDITOR_LINK_TYPE_CHOICES = getattr(
        settings, 'CKEDITOR_LINK_TYPE_CHOICES', (
            ('', _("None")),
            ('free', _("Other / Free")),
            ('external', _("External URL")),
            ('mailto', _("E-Mail")),
            ('phone', _("Phone")),
        )
    )



from django import forms
from django.contrib import admin

from ckeditor_link.admin import DjangoLinkAdmin
from .models import TestModel, LinkModel, ContribLinkModel, CMSFilerLinkModel

# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.forms.extras.widgets import SelectDateWidget
else:
    from django.forms.widgets import SelectDateWidget


class LinkModelForm(forms.ModelForm):
    when = forms.DateField(widget=SelectDateWidget, required=False)


admin.site.register(TestModel)


@admin.register(LinkModel)
class CKLinkModelAdmin(DjangoLinkAdmin):
    form = LinkModelForm


@admin.register(ContribLinkModel)
class CKBaseLinkModelAdmin(DjangoLinkAdmin):
    pass


@admin.register(CMSFilerLinkModel)
class CMSFilerCKLinkModelAdmin(DjangoLinkAdmin):
    pass



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-11-28 14:58
from __future__ import unicode_literals

import ckeditor.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='testmodel',
            name='richtext_second',
            field=ckeditor.fields.RichTextField(default=''),
        ),
        migrations.AlterField(
            model_name='testmodel',
            name='richtext',
            field=ckeditor.fields.RichTextField(default=''),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-11-28 14:54
from __future__ import unicode_literals

import ckeditor.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='LinkModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('target', models.CharField(blank=True, default='', max_length=255)),
                ('external_url', models.CharField(blank=True, default='', max_length=255)),
                ('email', models.EmailField(blank=True, default='', max_length=254)),
                ('when', models.DateField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255)),
                ('richtext', ckeditor.fields.RichTextField()),
            ],
        ),
        migrations.AddField(
            model_name='linkmodel',
            name='testmodel',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='test_app.TestModel'),
        ),
    ]



from django.conf import settings


CKEDITOR_LINK_MODEL = getattr(
    settings,
    'CKEDITOR_LINK_MODEL',
    None
)


# a default, working with the provided contrib link_model
CKEDITOR_LINK_ATTR_MODIFIERS = getattr(
    settings,
    'CKEDITOR_LINK_ATTR_MODIFIERS', {
        'cms_page': '{cms_page_2}'
    }
)



from __future__ import unicode_literals

from django.core.exceptions import ObjectDoesNotExist

try:
    from builtins import str
except ImportError:
    from __builtin__ import str

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from .conf import (
    CKEDITOR_LINK_TYPE_CHOICES,
    CKEDITOR_LINK_USE_CMS_FILER,
)

# dropped in favour of builtins/str, see above
# try:
#     unicode('')
# except NameError:
#     unicode = str


class LinkBase(models.Model):
    # name = models.CharField(
    #     verbose_name=_('Link Text'),
    #     max_length=255,
    #     blank=True,
    #     default='',
    # )
    link_type = models.CharField(
        max_length=20,
        blank=True,
        default='',
        verbose_name=_('Link type'),
    )
    link_style = models.CharField(
        max_length=64,
        blank=True,
        default='',
        verbose_name=_('Link Style'),
    )
    free = models.CharField(
        max_length=512,
        default='',
        blank=True,
        verbose_name=_("Internal / Other"),
    )
    external = models.URLField(
        blank=True,
        default='',
        verbose_name=_("External Address"),
    )
    mailto = models.EmailField(
        default='',
        blank=True,
        verbose_name=_("E-Mail"),
    )
    phone = models.CharField(
        max_length=64,
        default='',
        blank=True,
        verbose_name=_("Phone"),
    )

    class Meta:
        abstract = True

    def get_link(self):
        # if link type set, give priority!
        # this will alter your object!
        if self.link_type:
            for key, display in CKEDITOR_LINK_TYPE_CHOICES:
                if not key == self.link_type:
                    setattr(self, key, None)
        # generic foreign key link check
        fk_link = self._check_link_for_foreign_key()
        if fk_link:
            return fk_link
        # other custom link types
        if self.free:
            return self.free
        elif self.external:
            link = self.external
            if not link.startswith('http'):
                link = 'http://%s' % link
            return link
        elif self.mailto:
            return "mailto:%s" % self.mailto
        elif self.phone:
            return "tel:%s" % self.phone
        return ''

    def get_link_text(self):
        obj = None
        if getattr(self, 'link_text', None):
            return self.link_text
        if getattr(self, 'name', None):
            return self.name
        if self.link_type:
            obj = self._check_link_for_foreign_key()
        if object is not None:
            return str(obj)
        return ''

    def get_link_type(self):
        return self.link_type

    def get_link_style(self):
        return self.link_style

    def get_link_target(self):
        type = self.get_link_type()
        if type in ['file', 'external']:
            return "_blank"
        return ""

    def _check_foreign_key_value(self):
        """
        check if link type and it's value are a foreign key
        Returns: foreign key value, if available
        """
        try:
            value = getattr(self, self.get_link_type(), None)
            return value
        except ObjectDoesNotExist:
            pass
        return None

    def _check_link_for_foreign_key(self):
        value = self._check_foreign_key_value()
        # print(isinstance(value, models.Model))
        if value and getattr(value, 'get_absolute_url', None):
            link = value.get_absolute_url()
            return link
        return None


class Link(LinkBase):
    """
    if in installed apps, this will be created and available out of the box
    beware: no migrations yet!
    """
    pass


if CKEDITOR_LINK_USE_CMS_FILER:

    from cms.models.fields import PageField
    from filer.fields.file import FilerFileField

    class CMSFilerLinkBase(LinkBase):  # noqa
        cms_page = PageField(
            null=True,
            on_delete=models.SET_NULL,
            related_name="%(app_label)s_%(class)s_set",
            blank=True,
        )
        html_anchor = models.SlugField(
            default='',
            blank=True,
            verbose_name='Anker',
        )
        file = FilerFileField(
            null=True,
            on_delete=models.SET_NULL,
            related_name="%(app_label)s_%(class)s_set",
            blank=True,
        )

        def __init__(self, *args, **kwargs):
            if kwargs.get('page', None):
                self.cms_page = kwargs.get('page')
            super(CMSFilerLinkBase, self).__init__(*args, **kwargs)

        class Meta:
            abstract = True

        def get_link(self):
            # best practice is to call super first, so not relevant attrs are nulled
            super_link = super(CMSFilerLinkBase, self).get_link()
            fk_obj = self._check_foreign_key_value()
            if self.get_link_type() == 'cms_page' and super_link:
                if self.html_anchor:
                    super_link += '#%s' % self.html_anchor
                if getattr(self.cms_page, 'node', None):
                    # cms>=3.5
                    site = getattr(self.cms_page.node, 'site', None)
                else:
                    # cms<3.5
                    site = getattr(self.cms_page, 'site', None)
                if site.id == settings.SITE_ID:
                    return super_link
                else:
                    return '//' + site.domain + super_link
            elif self.get_link_type() == 'file' and fk_obj:
                return fk_obj.url
            return super_link

        def get_link_target(self):
            fk_obj = self._check_foreign_key_value()
            if self.get_link_type() == 'cms_page' and fk_obj:
                site = self._get_cms_page_site()
                if not site.id == settings.SITE_ID:
                    return "_blank"
            else:
                return super(CMSFilerLinkBase, self).get_link_target()
            return ""

        def _get_cms_page_site(self):
            fk_obj = self._check_foreign_key_value()
            if self.get_link_type() == 'cms_page' and fk_obj:
                if getattr(self.cms_page, 'node', None):
                    site = self.cms_page.node.site
                else:
                    site = self.cms_page.site
            return site



# -*- coding: utf-8 -*-
from django.test import Client
from django.test import TestCase

from ckeditor_link.tests.test_app.models import TestModel

# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse


class CKeditorLinkTemplateTagTests(TestCase):
    fixtures = ['test_app.json', ]

    def setUp(self):
        self.test_object = TestModel.objects.get(pk=2)
        self.test_object_not_existing_fk = TestModel.objects.get(pk=1)

    def tearDown(self):
        pass

    def test_tag_link_target_class_value(self):
        """
        does it transform everything as it should?
        """
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object.id])
        response = client.get(url)
        # check it!
        self.assertEqual(response.status_code, 200)

    def test_tag_no_destruction_of_existing_links(self):
        """
        normal existing <a href="xx" should not be tinkered with
        """
        pass

    def test_tag_robustness(self):
        """
        can it handle LinkModel with for example no get_css_class method?
        """
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object.id])
        response = client.get(url)
        # check it!
        self.assertEqual(response.status_code, 200)

    def test_not_existing_foreign_key_value(self):
        """
        can it handle no more existing foreign key values set in links?
        """
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object_not_existing_fk.id])
        response = client.get(url)
        # no exception, there we go!
        self.assertEqual(response.status_code, 200)

    def test_get_link_attrs(self):
        """
        can it output correct link attrs
        """
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object.id])
        response = client.get(url)
        # no exception, there we go!
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'data-test="abc"')

    def test_tag_attr_modifiers(self):
        """
        do attr modifiers work?
        has data-target="targetvalue", needs <a href="targetvalue--xy"
        """
        client = Client()
        url = reverse('testmodel_detail', args=[self.test_object.id])
        response = client.get(url)
        self.assertContains(response, 'href="targetvalue--xy"')



# -*- coding: utf-8 -*-
from time import sleep

from selenium.common.exceptions import NoSuchElementException

from ckeditor_link.tests.utils.selenium_utils import SeleniumTestCase
from ckeditor_link.tests.test_app.models import TestModel


# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse


class ckeditor_linkEditorTests(SeleniumTestCase):
    fixtures = ['test_app.json', ]

    def setUp(self):
        self.existing = TestModel.objects.get(pk=1)
        super(ckeditor_linkEditorTests, self).setUp()

    def tearDown(self):
        self.webdriver.quit()

    def test_app_index_get(self):
        # if this fails, everything is probably broken.
        self.login()
        self.open(reverse('admin:index'))
        self.webdriver.find_css(".app-test_app")

    def test_editor_has_button_dialog_opens_has_form(self):
        self.login()
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))
        # wait = WebDriverWait(self.webdriver, 5)
        # element = wait.until(EC.frame_to_be_available_and_switch_to_it((By.CSS_SELECTOR, '.cke_dialog_ui_html')))
        # WebDriverWait(self.webdriver, 4).until(EC.presence_of_element_located(By.CSS_SELECTOR, '.cke_dialog_ui_html'))
        sleep(1)  # #1 may ckeditor be very slow, so the click event is not handled?!
        button = self.webdriver.wait_for_css(".cke_button__djangolink")
        button[0].click()
        self.webdriver.wait_for_css(".cke_dialog_title")
        # sleep(2)  # argh
        iframe = self.webdriver.find_css(".cke_dialog_ui_html")
        self.webdriver.switch_to.frame(iframe)
        self.webdriver.wait_for_css("#id_target")

    def test_dialog_form_validation(self):
        self.login()
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))
        sleep(1)  # #1
        button = self.webdriver.wait_for_css(".cke_button__djangolink")
        button[0].click()
        self.webdriver.wait_for_css(".cke_dialog_title")
        # sleep(2)  # argh
        iframe = self.webdriver.find_css(".cke_dialog_ui_html")
        self.webdriver.switch_to.frame(iframe)
        email = self.webdriver.wait_for_css("#id_email")
        email.send_keys('what-foo')
        self.webdriver.switch_to.default_content()
        ok = self.webdriver.wait_for_css(".cke_dialog_ui_button_ok")
        ok.click()
        self.webdriver.switch_to.frame(iframe)
        self.webdriver.wait_for_css(".field-email .errorlist")
        email.send_keys('root@example.com')
        self.webdriver.switch_to.default_content()
        ok = self.webdriver.wait_for_css(".cke_dialog_ui_button_ok")
        ok.click()
        sleep(1)  # argh
        try:
            # check that dialog is gone or invisible
            title = self.webdriver.find_css(".cke_dialog_title")
            self.assertFalse(title.is_displayed())
        except NoSuchElementException:
            # ok if removed from the DOM!!
            pass

    def test_no_fake_null_as_string_values(self):
        self.login()
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))
        sleep(1)  # argh
        button = self.webdriver.wait_for_css(".cke_button__djangolink")
        button[0].click()
        self.webdriver.wait_for_css(".cke_dialog_title")
        sleep(1)  # argh
        iframe = self.webdriver.find_css(".cke_dialog_ui_html")
        self.webdriver.switch_to.frame(iframe)
        self.webdriver.wait_for_css("#id_testmodel")  # just wait for it, then send with "null"
        self.webdriver.switch_to.default_content()
        ok = self.webdriver.wait_for_css(".cke_dialog_ui_button_ok")
        ok.click()
        sleep(1)  # argh
        # inserted at first position in html, so this should work.
        ckcontent_iframe = self.webdriver.find_css("#cke_id_richtext iframe")
        self.webdriver.switch_to.frame(ckcontent_iframe)
        link = self.webdriver.wait_for_css("body a[data-ckeditor-link=true]")
        self.assertEqual('', link.get_attribute('data-testmodel'))

    def test_two_editors_in_same_form(self):
        # TODO
        self.login()
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))

    def test_dialog_submit_and_link_attrs(self):
        # TODO
        self.login()
        self.open(reverse('admin:test_app_testmodel_change', args=[self.existing.id]))



import importlib

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist

from ckeditor_link import conf
from django import template
from django.template.defaultfilters import stringfilter


try:
    module_name, class_name = conf.CKEDITOR_LINK_MODEL.rsplit(".", 1)
    my_module = importlib.import_module(module_name)
    ckeditor_link_class = getattr(my_module, class_name, None)
except ImportError:
    ckeditor_link_class = None


register = template.Library()


@register.filter
@stringfilter
def ckeditor_link_add_links(html):
    # lxml is not a dependency, but needed for this tag.
    from lxml.html import fragment_fromstring, tostring
    if not ckeditor_link_class:
        # TODO: use some log thing, or rais ImproperlyConfigured!
        if settings.DEBUG:
            msg = "Warning: CKEDITOR_LINK_MODEL (%s) could not be imported!?" % (conf.CKEDITOR_LINK_MODEL, )
            raise ImproperlyConfigured(msg)
        return html
    fragment = fragment_fromstring("<div>" + html + "</div>")
    links = fragment.cssselect('a')
    for link in links:
        if link.get('data-ckeditor-link', None):
            link.attrib.pop('data-ckeditor-link')
            kwargs = {}
            dummy_link = ckeditor_link_class()
            for key, value in link.items():
                if key.startswith('data-'):
                    new_key = key.replace('data-', '', 1)
                    # DEPRECATED: use CKEDITOR_LINK_ATTR_MODIFIERS setting!
                    if new_key == 'page_2':
                        new_key = 'cms_page'  # backward compat, for 0.2.0
                    if new_key == 'cms_page_2':
                        new_key = 'cms_page'
                    # until here
                    if hasattr(dummy_link, new_key):
                        if hasattr(dummy_link, new_key + "_id"):
                            # set fk directly
                            new_key = new_key + "_id"
                            if not value:
                                value = None
                        kwargs[new_key] = value
                        link.attrib.pop(key)
            for key, formatted_string in conf.CKEDITOR_LINK_ATTR_MODIFIERS.items():
                try:
                    kwargs[key] = formatted_string.format(**kwargs)
                except KeyError:
                    # this is an option, we dont know at all how our link is/was built (ages ago)
                    pass
            try:
                # this can go wrong with fk and the like
                real_link = ckeditor_link_class(**kwargs)
                link.set('href', real_link.get_link())
                if getattr(real_link, 'get_link_target', None):
                    link.set('target', real_link.get_link_target())
                if getattr(real_link, 'get_link_style', None):
                    link.set('class', real_link.get_link_style())
                if getattr(real_link, 'get_link_attrs', None):
                    for attr, value in real_link.get_link_attrs().items():
                        link.set(attr, value)
            except (ValueError, ObjectDoesNotExist):
                continue
    # arf: http://makble.com/python-why-lxml-etree-tostring-method-returns-bytes
    # beautifulsoup to the rescue!
    return tostring(fragment, encoding='unicode')



from django.conf import settings
from django.conf.urls import url
from django.contrib import admin
from django.db import models
from django.forms import widgets
from django.http import JsonResponse


class DjangoLinkAdmin(admin.ModelAdmin):

    def get_model_perms(self, request):
        """
        http://stackoverflow.com/questions/2431727/django-admin-hide-a-model
        Return empty perms dict thus hiding the model from admin index.
        """
        return {}

    @property
    def media(self):
        original_media = super(DjangoLinkAdmin, self).media
        css = {
            'all': (
                settings.STATIC_URL + 'admin/ckeditor_link/css/link_admin.css',
            )
        }
        new_media = widgets.Media(css=css)
        return original_media + new_media

    def get_urls(self):
        """
        add verify url.
        """
        my_urls = [
            url(
                r'^verify/$',
                self.admin_site.admin_view(self.verify),
                name=self._get_verify_url_name()
            ),
        ]
        return my_urls + super(DjangoLinkAdmin, self).get_urls()

    def _get_verify_url_name(self):
        return '{0}_{1}_verify'.format(self.model._meta.app_label,
                                       self.model._meta.model_name)

    def verify(self, request):
        """
        verify data with modelform, send through data.
        :param request:
        :return:
        """
        form = self.get_form(request, )(request.POST)
        if form.is_valid():
            verified_data = form.cleaned_data
            obj = self.model(**verified_data)
            link_value = ''
            # prepopulate href
            if (getattr(obj, 'get_link', None)):
                link_value = obj.get_link()
            # basic serialize only
            for key, value in verified_data.items():
                if isinstance(value, models.Model):
                    verified_data[key] = value.id
            return_data = {"valid": 'true', 'data': verified_data, 'link_value': link_value}
        else:
            errors = form.errors
            return_data = {"valid": 'false', 'errors': errors}
        return JsonResponse(return_data)

    def save_model(self, request, obj, form, change):
        """
        no save!
        """
        return False

    def get_changeform_initial_data(self, request):
        initial = super(DjangoLinkAdmin, self).get_changeform_initial_data(request)
        if request.GET.get('page', None):
            initial['cms_page'] = request.GET.get('page')
        return initial



from django.db import models
from ckeditor.fields import RichTextField
from ckeditor_link.link_model.models import CMSFilerLinkBase, LinkBase


# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse
else:
    from django.urls import reverse


class TestModel(models.Model):
    title = models.CharField(max_length=255, )
    richtext = RichTextField(default='')
    richtext_second = RichTextField(default='')

    def __str__(self):
        return "%s" % self.title

    def get_absolute_url(self):
        return reverse('testmodel_detail', args=(self.id, ))


# @python_2_unicode_compatible
# class CMSTestModel(models.Model):
#     title = models.CharField(max_length=255, )
#     richtext = HTMLField()
#
#     def __str__(self):
#         return "%s" % self.title


class LinkModelBase(models.Model):
    external_url = models.CharField(max_length=255, blank=True, default='',)
    email = models.EmailField(blank=True, default='',)
    # http://stackoverflow.com/questions/12644142/prefill-a-datetimefield-from-url-in-django-admin
    when = models.DateField(blank=True, null=True)
    testmodel = models.ForeignKey(
        TestModel,
        null=True,
        on_delete=models.CASCADE,
        default=None,
        blank=True,
    )
    target = models.CharField(
        max_length=255,
        blank=True,
        default='',
    )

    class Meta:
        abstract = True

    def __str__(self):
        return "LINK object: %s" % self.get_link()

    def get_link(self):
        # print(self.__dict__)
        if self.external_url:
            return self.external_url
        elif self.when:
            return self.when
        elif self.testmodel:
            return self.testmodel.get_absolute_url()
        elif self.target:
            return self.target
        else:
            return "http://no-link-given.com/"

    def get_link_target(self):
        return "_blank"

    def get_link_style(self):
        return "no-css-class"

    def get_link_attrs(self):
        return {'data-test': 'abc'}


class LinkModel(LinkModelBase):
    pass


class ContribLinkModel(LinkBase):
    pass


class CMSFilerLinkModel(CMSFilerLinkBase):
    pass



"""Settings that need to be set in order to run the tests."""
import os
# import sys
import tempfile
import logging

# compat
import django
if django.VERSION[:2] < (1, 10):
    from django.core.urlresolvers import reverse_lazy
else:
    from django.urls import reverse_lazy


DEBUG = True

logging.getLogger("factory").setLevel(logging.WARN)

HEADLESS_TESTING = True


CKEDITOR_LINK_USE_CMS_FILER = True

# not a good example, but working for our tests
CKEDITOR_LINK_ATTR_MODIFIERS = {
    'target': '{target}--xy',
}
CKEDITOR_LINK_MODEL = 'ckeditor_link.tests.test_app.models.LinkModel'
CKEDITOR_LINK_IFRAME_URL = reverse_lazy('admin:test_app_linkmodel_add')
CKEDITOR_LINK_VERIFY_URL = reverse_lazy('admin:test_app_linkmodel_verify')

CKEDITOR_CONFIGS = {
    'default': {
        'djangolinkIframeURL': CKEDITOR_LINK_IFRAME_URL,
        'djangolinkVerifyURL': CKEDITOR_LINK_VERIFY_URL,
        'djangolinkFallbackField': 'external_url',
        'linkShowAdvancedTab': False,
        'extraPlugins': ','.join(
            [
                # your extra plugins here
                'djangolink',
                'autolink',
                'autoembed',
                'embedsemantic',
                'autogrow',
                'devtools',
                'widget',
                'lineutils',
                'clipboard',
                'dialog',
                'dialogui',
                'elementspath'
            ]),
        'toolbar': 'Custom',
        'toolbar_Custom': [
            ['Bold', 'Underline'],
            ['DjangoLink', 'Unlink'],
            ['Link', 'Unlink'],
            ['RemoveFormat', 'Source']
        ]
    }
}

CMS_TEMPLATES = (
    ('cms_dummy.html', 'Dummy'),
)

SITE_ID = 1

APP_ROOT = os.path.abspath(
    os.path.join(os.path.dirname(__file__), ".."))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite',
    }
}

LANGUAGE_CODE = 'en'
LANGUAGES = (
    ('en', 'ENGLISHS', ),
)

X_FRAME_OPTIONS = 'SAMEORIGIN'
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'
ROOT_URLCONF = 'ckeditor_link.tests.urls'

# media root is overridden when needed in tests
MEDIA_ROOT = tempfile.mkdtemp(suffix='ckeditor_media_root')
MEDIA_URL = "/media/"
STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(APP_ROOT, '../test_app_static')
STATICFILES_DIRS = (
    os.path.join(APP_ROOT, 'static'),
)

# TEMPLATE_DIRS = (
#     os.path.join(APP_ROOT, 'tests/test_app/templates'),
# )

COVERAGE_REPORT_HTML_OUTPUT_DIR = os.path.join(
    os.path.join(APP_ROOT, 'tests/coverage'))
COVERAGE_MODULE_EXCLUDES = [
    'tests$', 'settings$', 'urls$', 'locale$',
    'migrations', 'fixtures', 'admin$', 'django_extensions',
]

EXTERNAL_APPS = (
    'django.contrib.admin',
    # 'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'django.contrib.sites',
    'ckeditor',
    # 'djangocms_text_ckeditor',
    'cms',
    'treebeard',
    'menus',
    'filer',
    'sekizai',
    'easy_thumbnails',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',
                'django.template.context_processors.request',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'sekizai.context_processors.sekizai',
                'cms.context_processors.cms_settings',
            ],
            'loaders': [
                'django.template.loaders.filesystem.Loader',
                'django.template.loaders.app_directories.Loader',
                # 'django.template.loaders.eggs.Loader',
            ],
        }
    },
]


INTERNAL_APPS = (
    'ckeditor_link',
    # 'ckeditor_link',
    'ckeditor_link.tests.test_app',
)

MIDDLEWARE = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    # Uncomment the next line for simple clickjacking protection:
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
)

INSTALLED_APPS = EXTERNAL_APPS + INTERNAL_APPS
COVERAGE_MODULE_EXCLUDES += EXTERNAL_APPS

SECRET_KEY = 'foobarXXXxxsvXY'



# coding: utf-8
from setuptools import setup, find_packages
import os


# not so bad: http://joebergantine.com/blog/2015/jul/17/releasing-package-pypi/
version = __import__('ckeditor_link').__version__


def read(fname):
    # read the contents of a text file
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="django-ckeditor-link",
    version=version,
    url='http://github.com/bnzk/django-ckeditor-link',
    license='MIT',
    platforms=['OS Independent'],
    description=read('DESCRIPTION'),
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    author=u'Ben Stähli',
    author_email='bnzk@bnzk.ch',
    packages=find_packages(),
    install_requires=(
        # 'Django>=1.3,<1.5',  # no need to limit while in development
        'Django>=1.11',
    ),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
)



# Generated by Django 2.2.15 on 2021-04-12 07:50

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0003_cmsfilerlinkmodel_contriblinkmodel'),
    ]

    operations = [
        migrations.AddField(
            model_name='cmsfilerlinkmodel',
            name='html_anchor',
            field=models.SlugField(blank=True, default='', verbose_name='Anker'),
        ),
        migrations.AlterField(
            model_name='cmsfilerlinkmodel',
            name='cms_page',
            field=cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='test_app_cmsfilerlinkmodel_set', to='cms.Page'),
        ),
        migrations.AlterField(
            model_name='cmsfilerlinkmodel',
            name='file',
            field=filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='test_app_cmsfilerlinkmodel_set', to='filer.File'),
        ),
    ]
